
--[[

name:   Antix Button
vers:   1.0.0
desc:   An image button that has seperate images for up and down states
auth:   by Cliff Earl, Antix Software
date:   June 2016
legal:  Distribute freely but please leave my credits intact.

notes:  Still needs a lot of work

--]]

Button = Core.class(Bitmap)

function Button:init(upTexture, downTexture, options) -- GoGo
  
  local props = {
    name            = "Button",
    x               = 0,
    y               = 0,
    handleByCenter  = true,
    scale           = 1,
    alpha           = 1,
    rotation        = 0,
    enabled         = true,
    onPressed       = nil,
    onDragged       = nil,
    onReleased      = nil,
  }
  
	if options then
		for key, value in pairs(options) do
			props[key]= value
		end
	end
  
  self.props = props
  
  self.parent = self -- For when using as a standalone button
  
  self.enabled = props.enabled
  self.name = props.name
  
  self.upTexture = upTexture
  self.downTexture = downTexture
  
  self:setScale(props.scale)
  
  self.focus = false
  self.id = -1

  if props.handleByCenter then self:setAnchorPoint(0.5, 0.5) end
  
  self:setPosition(props.x, props.y)
  self:setAlpha(props.alpha)
  self:setRotation(props.rotation)
  
  self:addEventListener(Event.TOUCHES_BEGIN, self.pressed, self) -- ADD EVENT HANDLERS
  self:addEventListener(Event.TOUCHES_MOVE, self.dragged, self)
  self:addEventListener(Event.TOUCHES_END, self.released, self)
end

function Button:dispose() -- Cleanup
  self.enabled = false
  self:removeEventListener(Event.TOUCHES_BEGIN, self.pressed, self) -- REMOVE EVENT HANDLERS
  self:removeEventListener(Event.TOUCHES_MOVE, self.dragged, self)
  self:removeEventListener(Event.TOUCHES_END, self.released, self)
  
  self.parent = nil
  self.enabled = nil
  self.focus = nil
  self.id = nil
end

function Button:enable(state) -- Enable / Disable
  self.enabled = state
end

function Button:pressed(e) -- Handle touch events
  if self:hitTestPoint(e.touch.x, e.touch.y) then
    e:stopPropagation()
    self.id = e.touch.id
    self.focus = true
    self.held = true
    local touchX, touchY = e.touch.x - self:getX(), e.touch.y - self:getY()
    self.touchX, self.touchY = touchX, touchY
    local props = self.props
    --print("Button Pressed.. "..self.name)
    if self.downTexture then self:setTextureRegion(self.downTexture) end
    if props.onPressed then
      props.onPressed(self) -- Perform action
    end
  end
end

function Button:dragged(e) -- Handle move events
  if e.touch.id == self.id then
    self.focus = true
    e:stopPropagation()
    
    local touchX, touchY = e.touch.x - self:getX(), e.touch.y - self:getY()
    self.touchX, self.touchY = touchX, touchY
    
    local props = self.props
    if props.onDragged then
      props.onDragged(self) -- Perform action
    end
   end
end

function Button:released(e) -- Handle touch end events
  local props = self.props
  self.held = false
  self:setTextureRegion(self.upTexture)
  if e.touch.id == self.id and self.focus then
    e:stopPropagation()
    self.focus = false
    self.id = -1
    --if self:hitTestPoint(e.touch.x, e.touch.y) then
      if props.onReleased then
        props.onReleased(self) -- Perform action
      end
    --end
  end
end
